//Required
#include "App.h"

//Using
#include "Ball.h"
#include "Flip.h"

using namespace Leadwerks;

App::App() : window(NULL), context(NULL), world(NULL), camera(NULL) {}

App::~App() { delete world; delete window; }

Vec3 camerarotation;
PickInfo pick;

bool App::Start()
{
	//Create a window
	int sWidth = 1024;
	int sHeight = 768;
	window = Window::Create("Bouncy Ball", GetSystemMetrics(0)/2-sWidth/2, GetSystemMetrics(1)/2-sHeight/2, sWidth, sHeight);
	
	//Create a context
	context = Context::Create(window);
	
	//Create a world
	world = World::Create();
	level = new Level();
	level->CreateLevel(0); //Start with menu
	
	//Create a camera
	camera = Camera::Create();
	camera->Move(0,0,-15);
	camera->SetProjectionMode(Camera::Orthographic);
	camera->SetZoom(18);

	//Set up the HUD
	hud = new HUD(window, context, camera, level);

	//Create the menu
	menu = new MenuManager(window, context, world, level, hud);
	menu->LoadSplash();
	
	string mapname = System::GetProperty("map","Maps/menu.map"); //level1
	Map::Load(mapname);
	
	//Move the mouse to the center of the screen
	window->SetMousePosition(context->GetWidth()/2,context->GetHeight()/2);
	
	return true;
}

bool App::Loop()
{
	//Close the window to end the program
	if (window->Closed()) return false;
    
	if (menu->GetRecreateCam()){
		cout << "RECREATING CAMERA -----------" << endl;
		//Create a camera
		camera = Camera::Create();
		camera->Move(0,0,-15);
		camera->SetProjectionMode(Camera::Orthographic);
		camera->SetZoom(18);
		menu->SetRecreateCam(false);
		hud->ChangeCamera(camera);
	}

	//Player input
	//cout << "Checking pick" << endl;
	if (window->MouseHit(1)){
		cout << "Cam check" << endl;
		//cout << "Cam zoom: " << camera->GetZoom() << endl;
		//cout << "Cam pos XYZ: " << camera->GetPosition().x << ", " << camera->GetPosition().y << ", " << camera->GetPosition().z << endl;
		camera->Pick(window->GetMousePosition().x, window->GetMousePosition().y, pick, 0, true);

		if (pick.entity != NULL){
			cout << "Pick ID: " << std::atoi(pick.entity->GetKeyValue("name").c_str()) << endl;
			//Flip
			if (std::atoi(pick.entity->GetKeyValue("name").c_str()) >= 10){
				cout << "It's a level object!" << endl;
				LevelObject* lo = level->GetLevelObject(std::atoi(pick.entity->GetKeyValue("name").c_str())-10);
				//cout << "LO's Pos X:" << lo->GetModel()->GetPosition(true).x << endl;

				//Do stuff with the object we clicked on
				Flip* f;
				Ball* b;

				switch (lo->GetObjectType()){
					case 1:
						cout << "It's a flip flop thingy!" << endl;
						f = (Flip*) lo;
						f->Rotate();
						lo->SetActive(!lo->GetActive());
						//level->UpdatePhysics();
					break;

					case 2:
						cout << "It's a ball!" << endl;
						b = (Ball*) lo;
						b->MoveToStartPos();
						//level->UpdatePhysics();
					break;

					case 4:
						cout << "Collect these!" << endl;
					break;

					default:
						lo->SetActive(!lo->GetActive());
					break;
				}
			}
		}
	}

	//cout << "Updating level" << endl;
	level->Update();

	Time::Update();
	/*if (camera == NULL){
		cout << "CAMERA IS NULL!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << endl;
	}

	if (world == NULL || world->GetSize() == 0){
		cout << "CAMERA IS NULL!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << endl;
	}*/
	world->GetCurrent()->Update();
	world->GetCurrent()->Render();

	//cout << "Drawing HUD / Menu" << endl;
	context->SetBlendMode(Blend::Alpha);
	hud->Draw();
	menu->Draw();
	context->SetBlendMode(Blend::Solid);

	context->Sync(false);
	
	return true;
}
