#include "HUD.h"
#include "LevelObject.h"
#include "Score.h"

PickInfo pickinfo;

HUD::HUD(Window* window, Context* context, Camera* camera, Level* level){
	this->window = window;
	this->context = context;
	this->camera = camera;
	this->level = level;

	showedPointsFeedback = false;
	feedbackAlpha = 1;
	isIngame = false;
	wonGame = false;

	//Feedback images
	youWin = Texture::Load("Materials/YouWin.tex");
	pointsCollected = Texture::Load("Materials/PointsCollected.tex");

	//Interaction
	canInteract = Texture::Load("Materials/CanInteract.tex");
	canNotInteract = Texture::Load("Materials/CanNotInteract.tex"); //Objects will not interact with it

	//Steps
	StepCompleted = Texture::Load("Materials/StepDone.tex");;
	Step1Unlocked = Texture::Load("Materials/Step1Unlocked.tex");

	Step2Unlocked = Texture::Load("Materials/Step2Unlocked.tex");
	Step2Locked = Texture::Load("Materials/Step2Locked.tex");
}

HUD::~HUD(){
	youWin->Release();
	pointsCollected->Release();
	canInteract->Release();
	canNotInteract->Release();
}

void HUD::ChangeCamera(Camera* camera){
	cout << "Changing camera of the HUD" << endl;
	this->camera = NULL;
	this->camera = camera;
}

void HUD::SetIngame(bool val){
	isIngame = val;
	curStep = 1;
	cout << "Game started." << endl;
}

bool HUD::PointsCollected(){
	for (int i = 0; i < level->GetObjects().size(); i++){
		//objects.at(i)->GetModel()->UpdatePhysics();

		//Ball?
		if (level->GetLevelObject(i)->GetObjectType() == 4){
			Score* s = (Score*) level->GetLevelObject(i);
			if (s->GetActive()){
				//cout << "Point " << i << " still active." << endl;
				return false; //Still a point active
			}
		}
	}

	//No more point active
	//cout << "Level cleared!" << endl;
	return true;
}

bool HUD::BallsAreDown(){
	for (int i = 0; i < level->GetObjects().size(); i++){
		if (level->GetLevelObject(i)->GetObjectType() == 2){
			if (level->GetLevelObject(i)->GetModel()->GetPosition(true).y > -13.625){
				return false;
			}
		}
	}

	return true;
}

void HUD::Draw(){
	if (wonGame){
		youWin->Draw(window->GetWidth()/2 - youWin->GetWidth()/2, window->GetHeight()*.3f - youWin->GetHeight()/2);

		if (window->KeyHit(Key::Space)){
			//Go back to main menu
		}
		return;
	}

	camera->Pick(window->GetMousePosition().x, window->GetMousePosition().y, pickinfo, 0, true);

	//Draw step to step list
	if (isIngame){
		//Step 1
		Step1Unlocked->Draw(0, window->GetHeight()*.3f);

		//Draw 'Done' if done
		if (curStep >= 2)
			StepCompleted->Draw(0, window->GetHeight()*.3f);

		//Check if done if it wasn't already
		else if	(curStep == 1 && PointsCollected())
			curStep ++;

		//Step 2
		if (curStep >= 2) //Unlocked
			Step2Unlocked->Draw(0, window->GetHeight()*.3f+128);
		else //Locked
			Step2Locked->Draw(0, window->GetHeight()*.3f+128);

		//Draw 'Done' if done
		if (curStep >= 3)
			StepCompleted->Draw(0, window->GetHeight()*.3f+128);
		else if (curStep == 2 && BallsAreDown())
			curStep ++;

		if (curStep > 2)
			youWin->Draw(window->GetWidth()/2 - youWin->GetWidth()/2, window->GetHeight()/2 - youWin->GetHeight()/2);
	}

	if (pickinfo.entity != NULL){
		//cout << "Pick ID: " << std::atoi(pick.entity->GetKeyValue("name").c_str()) << endl;
		//Flip
		if (std::atoi(pickinfo.entity->GetKeyValue("name").c_str()) >= 10){
			//cout << "It's a level object!" << endl;
			LevelObject* lo = level->GetLevelObject(std::atoi(pickinfo.entity->GetKeyValue("name").c_str())-10);
			//cout << "LO's Pos X:" << lo->GetModel()->GetPosition(true).x << endl;

			//Draw feedback
			if (lo->GetActive())
				canInteract->Draw(window->GetMousePosition().x+32 -canInteract->GetWidth()/2, window->GetMousePosition().y+32 -canInteract->GetHeight()/2);
			else
				canNotInteract->Draw(window->GetMousePosition().x+32 -canNotInteract->GetWidth()/2, window->GetMousePosition().y+32 -canNotInteract->GetHeight()/2);
		}
	}
}